//////////////////////////////////////////////////////////////////
//Fluchtdiagramm der Standardmap in Polarkoordinaten
//Methode: Es wird bei hier 100 Iterationen die Standardabweichung
//der jeweiligen Winkelabstände zwischen zwei benachbarten Folgegliedern
//zur Codierung der Farbe verwendet.
//Mit + -Taste zweifache Vergrößerung .
//Zuvor mit Linksklick die interessierende Stelle zentrieren.
/////////////////////////////////////////////////////////////////


int iterations = 150; //Anzahl der Iterationen
double K = 0.971635;
//double K = 0.5;//Alternative
PImage bild, bildGedreht, bildZurueckGedreht;
//PGraphics hier zu verwenden, mag umständlich erscheinen. Wir nutzen es hier, um den Sketch einfach
//auf multithreaded umzuschreiben
PGraphics pg;
float vFaktor = 1  ;//Vergrößerungsfaktor
float fFaktor = 30; //Farbfaktor
double  xUNeu, xAlt, yAlt, xNeu, yNeu;
float xK, yK; //Koordinaten der Punkte
float alpha,r;
boolean zeichnen;
float mx, my;
double   xH, yH; 
float xKNeu, yKNeu;
double test =5000;
double [] abstand = new double[iterations];
double varianz =0;
double standardAbweichung =0;
double min = 500; double max = 0;


void setup() {
  background(255);
  colorMode(HSB);
  for(int i =0; i<iterations;i++){
    abstand[i] = 0;
  }
  zeichnen = true;  
  size(800, 800); 
  pg = createGraphics(width,height);
  bild = createImage(width,height,HSB);
  mx = 400; my = 400; //Mittelpunkt zu Beginn
  zeichneFluchtdiagramm();
 
}
//////////////////////////////////////////////////////////////
void draw() {

 if(zeichnen){  
   image(pg,0,0);
   zeichnen = false;     
   }
}
/////////////////////////////////////////////////////
void zeichneFluchtdiagramm(){
   pg.beginDraw();
      pg.loadPixels();
      
        r=0; alpha=0;
        for (int j = 0; j < height; j++) {          
          for (int i = 0; i < width; i++) {
            umrechnen(i,j, mx, my);//Umrechnung von (i,j) in( alpha,r)
            double D = standardMapIteration(alpha,r, iterations);
            pg.pixels[i+j*width] = colorize(D);                           
          }          
        }
         pg.updatePixels();     
    pg.endDraw(); 
    
}

/////////////////////////////////////////////////////////////
color colorize(double dist) {

  // jedes Pixel wird entsprechend der Standardabweichung bei der Iteration eingefärbt
   color col;  
   if(dist < 2) {
     col = color(0);//Markiert die elliptischen Fixpunkte
   }else{
   dist = dist%256;      
   col = color((int)(dist),255,255); 
   }  
   //alternativ:
   //int abdunkeln = ((int)((double)-1/64*dist*dist+4*dist) )% 256; 
   //col = color((int)(dist),255,abdunkeln);   
     
    return col;  
}
/////////////////////////////////////////////////////////////////////
double standardMapIteration(double xH, double yH, int maxN) {
  
  //Standardmap Algorithmus 
  //x steht für den Winkel, y für den Radius
  int  maxit = maxN;
  varianz =0;
  double d = 0;  
  xAlt = xH; yAlt = yH;
  int i = 0;
  while (i < maxit) {
    yNeu = (yAlt + K*Math.sin(xAlt)) ;  
    xNeu = (xAlt + yNeu) ;
    abstand[i]=  Math.abs(xNeu-xAlt);
    d = d+ Math.abs(xNeu-xAlt);
    xAlt = xNeu;yAlt = yNeu;   
    i++;
  }
  d = d/maxit; //Durchschnitt
  for(int j =0; j< maxit;j++){
    varianz = varianz + (abstand[j]-d)*(abstand[j]-d);
  }
  standardAbweichung = Math.sqrt(varianz);
  //if(standardAbweichung < min) min = standardAbweichung;
 // if(standardAbweichung > max) max = standardAbweichung;
  
  return fFaktor*standardAbweichung; //Invariante Kurven sieht man am besten mit fFaktor =20
}

/////////////////////////////////////////////////////////////////
void umrechnen(float x, float y, float xMP, float yMP){
  mx = xMP;
  my = yMP;
  xK =x;// - dx;
  yK =y;// - dy;
  if((xK >=mx && yK <= my) || (xK >= mx && yK >= my)){//1. und 4.Quadrant
    r = sqrt((xK-mx)*(xK-mx)+(yK-my)*(yK-my));
    alpha = asin((my-yK)/r);
  }
  
  if((xK <= mx && yK <= my) || (xK <= mx && yK >= my)){//2. und 3.Quadarant 
    r = sqrt((xK-mx)*(xK-mx)+(yK-my)*(yK-my));
    alpha = PI-asin((my-yK)/r);
    
  } 
  r = 20*r/width/vFaktor; 
}

/////////////////////////////////////////////////////////////////
void mousePressed() {
  xKNeu = mouseX;
  yKNeu = mouseY;
  if (mouseButton == LEFT) {       
     mx = mx-(xKNeu-width/2);
     my = my+(height/2-yKNeu) ;   
     zeichnen = true;
     zeichneFluchtdiagramm();    
  }
    
  if (mouseButton == RIGHT) {
     //what ever...
  }
}  
/////////////////////////////////////////////////////////////////
void keyPressed() {
  if(key == '+'){
     zeichnen = true;     
     vFaktor = 2*vFaktor; 
     mx = -width/2+2*mx;
     my = -height/2+2*my;
     
     println("Zoomfaktor: "+vFaktor+"-fach"); 
     zeichneFluchtdiagramm();
     
  }
  
  if(key == 'l'){
     zeichnen = true;     
     mx = mx -100;
     println("Verschiebung um 100 Pixel nach links"); 
     zeichneFluchtdiagramm();
  }
  if(key == 'r'){
     zeichnen = true;     
     mx = mx +100;
     println("Verschiebung um 100 Pixel nach rechts"); 
     zeichneFluchtdiagramm();
  }
  if(key == 'o'){
     zeichnen = true;     
     my = my -100;
     println("Verschiebung um 100 Pixel nach oben"); 
     zeichneFluchtdiagramm();
  }
  if(key == 'u'){
     zeichnen = true;     
     my = my +100;
     println("Verschiebung um 100 Pixel nach unten"); 
     zeichneFluchtdiagramm();
  }
  
  if (key == 's') {
    save("standardMapFlucht.png");
  }
  
  redraw();
}
/////////////////////////////////////////////////////////////////
